pub mod triangle;

use std::sync::Arc;
use vulkano::device::Device;
use vulkano::pipeline::GraphicsPipeline;

pub trait Pipeline {
    fn create_pipeline(device: &Arc<Device>) -> Arc<GraphicsPipeline>;
    fn get_pipeline(&self) -> &Arc<GraphicsPipeline>;
}

pub struct PipelineManager {
    pipelines: std::collections::HashMap<String, Arc<GraphicsPipeline>>,
}

impl PipelineManager {
    pub fn new() -> Self {
        Self {
            pipelines: std::collections::HashMap::new(),
        }
    }

    pub fn register_pipeline(&mut self, name: String, pipeline: Arc<GraphicsPipeline>) {
        self.pipelines.insert(name, pipeline);
    }

    pub fn get_pipeline(&self, name: &str) -> Option<&Arc<GraphicsPipeline>> {
        self.pipelines.get(name)
    }
}
