use std::sync::Arc;
use vulkano::descriptor_set::DescriptorSet;
use vulkano::descriptor_set::WriteDescriptorSet;
use vulkano::descriptor_set::allocator::StandardDescriptorSetAllocator;
use vulkano::descriptor_set::layout::{
    DescriptorSetLayout, DescriptorSetLayoutBinding, DescriptorSetLayoutCreateInfo, DescriptorType,
};
use vulkano::device::Device;
use vulkano::shader::ShaderStages;

pub struct DescriptorManager {
    device: Arc<Device>,
    allocator: Arc<StandardDescriptorSetAllocator>,
}

impl DescriptorManager {
    pub fn new(device: Arc<Device>, allocator: Arc<StandardDescriptorSetAllocator>) -> Self {
        Self { device, allocator }
    }

    pub fn create_descriptor_set_layout(
        &self,
        bindings: &[(u32, DescriptorType, u32)],
    ) -> Arc<DescriptorSetLayout> {
        let mut bindings_map = std::collections::BTreeMap::new();
        for (binding_index, ty, _count) in bindings {
            let mut binding = DescriptorSetLayoutBinding::descriptor_type(*ty);
            binding.stages = ShaderStages::all_graphics();
            bindings_map.insert(*binding_index, binding);
        }

        let create_info = DescriptorSetLayoutCreateInfo {
            bindings: bindings_map,
            ..Default::default()
        };

        DescriptorSetLayout::new(self.device.clone(), create_info).unwrap()
    }

    pub fn create_descriptor_set(
        &self,
        layout: &Arc<DescriptorSetLayout>,
        writes: Vec<WriteDescriptorSet>,
    ) -> Arc<DescriptorSet> {
        DescriptorSet::new(self.allocator.clone(), layout.clone(), writes, []).unwrap()
    }
}
